import {Component, Input, OnInit} from '@angular/core';
import {InteractionSequences} from "../../../project/game-model/interactions/InteractionSequences";
import {Interaction} from "../../../project/game-model/interactions/Interaction";
import {MatTableDataSource} from "@angular/material/table";
import {GameModel} from "../../../project/game-model/GameModel";
import {Character} from "../../../project/game-model/characters/Character";
import {MatSnackBar} from "@angular/material/snack-bar";
import {AbstractInteraction} from "../../../project/game-model/interactions/AbstractInteraction";
import {ScriptAccountCondition} from "../../../project/game-model/gamesystems/conditions/ScriptAccountCondition";
import {ScriptAccountAction} from "../../../project/game-model/gamesystems/actions/ScriptAccountAction";

@Component({
  selector: 'app-interaction-sequence-editor',
  templateUrl: './interaction-sequence-editor.component.html',
  styleUrl: './interaction-sequence-editor.component.scss'
})
export class InteractionSequenceEditorComponent implements OnInit{
  @Input() interactionSequence: InteractionSequences | undefined;
  @Input() gameModel: GameModel | undefined
  @Input() character: Character | undefined
  currentInteractionNode: Interaction | undefined;

  sequenceDatasource = new MatTableDataSource<Interaction>()
  displayedColumns: string[] = ['source', 'target', 'label', 'edit', 'delete']
  columnsToDisplayWithExpand = [... this.displayedColumns, 'expand'];
  expandedElement: AbstractInteraction | null = null;
  editedElement: Interaction | undefined

  constructor(private snackbar: MatSnackBar) {
  }
  ngOnInit() {
    if(this.interactionSequence != undefined) {
      this.assignData();
    }
  }

  private assignData() {
    if(this.currentInteractionNode != undefined) {
      this.sequenceDatasource.data = this.interactionSequence!.findInteraction(this.currentInteractionNode)!.children.map(node => node.root);
    } else {
      this.sequenceDatasource.data = [this.interactionSequence!.rootInteraction.root];
    }
  }

  editInteraction(interaction: Interaction) {
    this.editedElement = interaction;
  }

  submitInteraction() {
    if(this.editedElement == undefined) {
      return;
    }

    if(this.editedElement!.validate(this.character!)) {
      this.gameModel!.addCharacterInteraction(this.editedElement);
      this.editedElement = undefined;
    } else {
      this.snackbar.open("Invalid Interaction", "", {duration: 2000});
    }
  }

  deleteInteraction(interaction: Interaction) {
    if(this.currentInteractionNode != undefined) {
      const node = this.interactionSequence?.findInteraction(this.currentInteractionNode);
      if(node != undefined) {
        node.removeChildInteraction(interaction);
        this.assignData();
      }
    }
  }

  addInteraction() {
    const interaction = new Interaction(this.character!, undefined, "")
    this.editedElement = interaction;

    const interactions = this.sequenceDatasource.data;
    interactions.push(interaction);
    this.sequenceDatasource.data = interactions;
  }

  onAddCondition(interaction: AbstractInteraction, condition: ScriptAccountCondition) {
    interaction!.addConditon(condition);
  }

  onDeleteCondition(interaction: AbstractInteraction, condition: ScriptAccountCondition) {
    interaction!.removeCondition(condition);
  }

  onAddAction(interaction: AbstractInteraction, action: ScriptAccountAction) {
    if(interaction instanceof Interaction) {
      interaction.addAction(action);
    }
  }
  onRemoveAction(interaction: AbstractInteraction, action: ScriptAccountAction) {
    if(interaction instanceof Interaction) {
      interaction.removeAction(action);
    }
  }
}
